#ifndef I2C_H
#define I2C_H

// TWSR values (not bits)
// (taken from avr-libc twi.h - thank you Marek Michalkiewicz)
// Master
#define TW_START				0x08
#define TW_REP_START				0x10
// Master Transmitter
#define TW_MT_SLA_ACK				0x18
#define TW_MT_SLA_NACK				0x20
#define TW_MT_DATA_ACK				0x28
#define TW_MT_DATA_NACK				0x30
#define TW_MT_ARB_LOST				0x38
// Master Receiver
#define TW_MR_ARB_LOST				0x38
#define TW_MR_SLA_ACK				0x40
#define TW_MR_SLA_NACK				0x48
#define TW_MR_DATA_ACK				0x50
#define TW_MR_DATA_NACK				0x58
// Misc
#define TW_NO_INFO				0xF8
#define TW_BUS_ERROR				0x00

// defines and constants
#define TWCR_CMD_MASK		                0x0F
#define TWSR_STATUS_MASK	                0xF8

// return values
#define I2C_OK				        0x00
#define I2C_ERROR_NODEV		                0x01

// types
typedef enum {I2C_IDLE=0,I2C_BUSY=1,I2C_MASTER_TX=2,I2C_MASTER_RX=3} i2c_states_t;

/* public functions */
void i2c_init(uint16_t bitrate_khz);
void i2c_set_bitrate(uint16_t bitrate_khz);
void i2c_master_send(uint8_t deviceAddr, uint8_t length, uint8_t *data);
void i2c_master_receive(uint8_t deviceAddr, uint8_t length);
void i2c_master_reg_receive(uint8_t deviceAddr,uint8_t reg_address, uint8_t length);
void i2c_get_data(unsigned char length, unsigned char *data);
i2c_states_t i2c_get_state(void);

#endif
