#include "dac.h"
#include "mem.h"

#define         DAC_TOP_VALUE         ((uint32_t)1600)
#define         DAC_PRESCALE_MASK     1
#define         DAC_MAX_VOLTAGE       ((uint32_t)40960) //5V in 3|13 fixed point format

/* private variables */
uint8_t dac_running;

/* public functions */
void dac_init(void)
{
  // configure as outputs the DA pins
  DDRB|=0x60;

  /* initialize internal variables */
  dac_running=0;
  OCR1AH=0x00;
  OCR1AL=0x00;
  OCR1BH=0x00;
  OCR1BL=0x00;
  /* set the top value */
  ICR1H=(DAC_TOP_VALUE>>8)&0xFF;
  ICR1L=(DAC_TOP_VALUE)&0xFF;
  
  /* configure the timer 3 */
  TCNT1H=0x00;
  TCNT1L=0x00;
  /* set the timer mode */
  TCCR1A=(1<<WGM11);// all three channels in fast PWM mode, output disabled
  TCCR1B=(1<<WGM13)|(1<<WGM12);// no clock source, fast PWM mode
}

void dac_start(void)
{
  TCNT1H=0x00;
  TCNT1L=0x00;
  /* set the prescaler value */
  TCCR1B&=0xF8;
  TCCR1B|=DAC_PRESCALE_MASK;
  dac_running=0x01;
  ram_data[DAC_CONTROL]|=0x10;
}

void dac_stop(void)
{
  /* set the prescaler to 0 (no clock source) */
  TCCR1B&=0xF8;
  dac_running=0x00;
  ram_data[DAC_CONTROL]&=0xEF;
}

void dac_enable_channel(dac_ch_t ch)
{
  switch(ch)
  {
    case dac0: TCCR1A|=(1<<COMA1);
               ram_data[DAC_CONTROL]|=0x40;
               break;
    case dac1: TCCR1A|=(1<<COMB1);
               ram_data[DAC_CONTROL]|=0x80;
               break;
  }
}

void dac_disable_channel(dac_ch_t ch)
{
  switch(ch)
  {
    case dac0: TCCR1A&=~(1<<COMA1);
               ram_data[DAC_CONTROL]&=0xBF;
               break;
    case dac1: TCCR1A&=~(1<<COMB1);
               ram_data[DAC_CONTROL]|=0x7F;
               break;
  }
}

void dac_set_voltage(dac_ch_t ch,uint16_t voltage)
{
  uint32_t compare_value;

  if(voltage>DAC_MAX_VOLTAGE)
    voltage=DAC_MAX_VOLTAGE;
  compare_value=(voltage*DAC_TOP_VALUE)/DAC_MAX_VOLTAGE;

  switch(ch)
  {
    case dac0: OCR1AH=(compare_value>>8)&0xFF;
               OCR1AL=(compare_value)&0xFF;
               ram_data[DAC_CH0_VOLTAGE_L]=voltage%256;
               ram_data[DAC_CH0_VOLTAGE_H]=voltage/256;
               break;
    case dac1: OCR1BH=(compare_value>>8)&0xFF;
               OCR1BL=(compare_value)&0xFF;
               ram_data[DAC_CH1_VOLTAGE_L]=voltage%256;
               ram_data[DAC_CH1_VOLTAGE_H]=voltage/256;
               break;
  }
}

