#include "dyn_common.h"
#include "dyn_slave_se.h"
#include "mem.h"
#include "ports.h"
#include "adc.h"
#ifdef _PWM
#include "pwm.h"
#endif
#ifdef _DAC
#include "dac.h"
#endif
#ifdef _BNO055
#include "bno055.h"
#else
#include "compass.h"
#endif
#ifdef _UART_TTL
#include "uart_ttl.h"
#endif
#ifdef _UART_USB
#include "uart_usb.h"
#endif
#include "pwm_dyn_emm.h"

uint8_t do_write(uint8_t address,uint8_t length,uint8_t *data)
{
  uint8_t i,num=0;
  uint8_t low_byte=0x00;

  for(i=address;i<address+length;i++)
  {
    /* dynamixel interface */
    if(i==ID)
      dyn_slave_se_set_id(data[num]);
    else if(i==Baud_Rate)
      dyn_slave_se_set_baudrate(data[num]);
    /* gpio interface */
    else if(i>=GPIO0_config && i<=GPIO18_data)
    {
      if((i%2)==0)// config register
        config_port((i-GPIO0_config)>>1,data[num]);
      else// data register
      {
        if(data[num]==0x00)
          clear_port((i-GPIO0_data)>>1);
        else
          set_port((i-GPIO0_data)>>1);
      }
    }
    else if(i==LED)
    { 
      if(data[num]==0x00)
        clear_led();
      else
        set_led();
    }
    /* ADC interface */
    else if(i==ADC_CONTROL)
    {
      if(data[num]&0x01)
        adc_start();
      else if(data[num]&0x02)
        adc_stop();
    }
    else if(i==ADC_NUM_SAMPLES)
      adc_set_num_samples(data[num]);
    else if(i==ADC_SAMPLE_PERIOD)
      adc_set_sample_period(data[num]);
    #ifdef _PWM
    /* PWM interface */
    else if(i==PWM_CONTROL)
    {
      if(data[num]&0x01)
        pwm_start();
      else if(data[num]&0x02)
        pwm_stop();
    }
    else if(i==PWM_ENABLE)
    {
      if(ram_data[PWM0_DYN_ID]==0xFF)// disable the normal operation if it is being used as a servo
      {
        if(data[num]&0x01)
          pwm_enable_ch(pwm0);
        else if(data[num]&0x10)
          pwm_disable_ch(pwm0);
      }
      if(ram_data[PWM1_DYN_ID]==0xFF)// disable the normal operation if it is being used as a servo
      {
        if(data[num]&0x02)
          pwm_enable_ch(pwm1);
        else if(data[num]&0x20)
          pwm_disable_ch(pwm1);
      }
      if(ram_data[PWM2_DYN_ID]==0xFF)// disable the normal operation if it is being used as a servo
      {
        if(data[num]&0x04)
          pwm_enable_ch(pwm2);
        else if(data[num]&0x40)
          pwm_disable_ch(pwm2);
      }
      if(ram_data[PWM3_DYN_ID]==0xFF)// disable the normal operation if it is being used as a servo
      {
        if(data[num]&0x08)
          pwm_enable_ch(pwm3);
        else if(data[num]&0x80)
          pwm_disable_ch(pwm3);
      }
    }  
    else if(i==PWM_FREQ_L)
      low_byte=data[num];
    else if(i==PWM_FREQ_H)
    {
      if(ram_data[PWM0_DYN_ID]==0xFF && ram_data[PWM1_DYN_ID]==0xFF && ram_data[PWM2_DYN_ID]==0xFF && ram_data[PWM3_DYN_ID]==0xFF)
        pwm_set_frequency(low_byte+data[num]*256);
    }
    else if(i>=PWM_CH0_DUTY && i<=PWM_CH3_DUTY) 
    {
      if(ram_data[PWM0_DYN_ID+(i-PWM_CH0_DUTY)]==0xFF)
        pwm_set_duty_cycle((pwm_id_t)(i-PWM_CH0_DUTY),data[num]);
    }
    #endif
    #ifdef _DAC
    /* dac interface */
    else if(i==DAC_CONTROL)
    {
      if(data[num]&0x01)
        dac_start();
      else if(data[num]&0x02)
        dac_stop();
    }
    else if(i==DAC_ENABLE)
    {
      if(data[num]&0x01)
        dac_enable_channel(dac0);
      else if(data[num]&0x04)
        dac_disable_channel(dac0);
      if(data[num]&0x02)
        dac_enable_channel(dac1);
      else if(data[num]&0x08)
        dac_disable_channel(dac1);
    }
    else if(i==DAC_CH0_VOLTAGE_L)
      low_byte=data[num];
    else if(i==DAC_CH0_VOLTAGE_H)
      dac_set_voltage(dac0,low_byte+data[num]*256);
    else if(i==DAC_CH1_VOLTAGE_L)
      low_byte=data[num];
    else if(i==DAC_CH1_VOLTAGE_H)
      dac_set_voltage(dac1,low_byte+data[num]*256);
    #endif
    /* compass interface */
    #ifdef _BNO055
    else if(i==IMU_CONTROL)
    {
      if(data[num]&0x01)
        bno055_start();
      else if(data[num]&0x02)
        bno055_stop();
      else if(data[num]&0x04)
        bno055_erase_calibration();
    }
    #else
    else if(i==COMPASS_CONTROL)
    {
      if(data[num]&0x01)
        compass_start();
      else if(data[num]&0x02)
        compass_stop();
    }
    else if(i==COMPASS_NUM_SAMPLES)
      compass_set_num_samples(data[num]);
    #endif
    #ifdef _UART_TTL
    /* uart TTL interface */
    else if(i==UART_TTL_CONTROL)
    {
      if(data[num]&0x01)
        uart_ttl_start();
      else if(data[num]&0x02)
        uart_ttl_stop();
    }
    else if(i==UART_TTL_TX_DATA)
      uart_ttl_send_byte(data[num]);
    else if(i==UART_TTL_BAUDRATE_L)
      low_byte=data[num];
    else if(i==UART_TTL_BAUDRATE_H)
      uart_ttl_set_baudrate(low_byte+data[num]*256);
    #endif
    #ifdef _UART_USB
    /* uart TTL interface */
    else if(i==UART_USB_CONTROL)
    {
      if(data[num]&0x01)
        uart_usb_start();
      else if(data[num]&0x02)
        uart_usb_stop();
    }
    else if(i==UART_USB_TX_DATA)
      uart_usb_send_byte(data[num]);
    else if(i==UART_USB_BAUDRATE_L)
      low_byte=data[num];
    else if(i==UART_USB_BAUDRATE_H)
      uart_usb_set_baudrate(low_byte+data[num]*256);
    #endif
    /* emmulated dynamixel motors */
    else if(i>=PWM0_DYN_ID && i<=PWM3_DYN_ID)
      pwm_dyn_set_dyn_id(i-PWM0_DYN_ID,data[num]);
    num++;
  }
  eeprom_write(address,length,data);

  return 0x01;
}

uint8_t do_read(uint8_t address,uint8_t length,uint8_t **data)
{
  uint8_t i=0;

  /* perform operations before reading the ram */
  for(i=address;i<address+length;i++)
  {
    if(i>=GPIO0_config && i<=GPIO18_data)
    {
      if((i%2)!=0) // data register
        read_port((i-GPIO0_data)>>1);
    }
    else if(i==SWITCHES)
      get_selector_value();
    #ifdef _UART_TTL
    else if(i==UART_TTL_CONTROL)
    {
      if(uart_ttl_is_data_available())
        ram_data[UART_TTL_CONTROL]|=0x80;
      else
        ram_data[UART_TTL_CONTROL]&=0x7F;
      if(uart_ttl_is_sending())
        ram_data[UART_TTL_CONTROL]|=0x40;
      else
        ram_data[UART_TTL_CONTROL]&=0xBF;
    }
    else if(i==UART_TTL_RX_DATA)
      uart_ttl_receive_byte(&ram_data[UART_TTL_RX_DATA]);
    #endif
    #ifdef _UART_USB
    else if(i==UART_USB_CONTROL)
    {
      if(uart_usb_is_data_available())
        ram_data[UART_USB_CONTROL]|=0x80;
      else
        ram_data[UART_USB_CONTROL]&=0x7F;
      if(uart_usb_is_sending())
        ram_data[UART_USB_CONTROL]|=0x40;
      else
        ram_data[UART_USB_CONTROL]&=0xBF;
    }
    else if(i==UART_USB_RX_DATA)
      uart_usb_receive_byte(&ram_data[UART_USB_RX_DATA]);
    #endif
  }
  return ram_read(address,length,data); 
}
