#include "exp_board_config.h"
#include <avr/interrupt.h>
#include <avr/io.h>
#include "dyn_slave_se.h"
#include "mem.h"
#include "pwm_dyn_emm.h"

#ifndef DYN_SLAVE_SE_BUFFER_LEN
  #define DYN_SLAVE_SE_BUFFER_LEN      128
#endif

/* private variables */
uint8_t dyn_slave_se_id;
uint8_t dyn_slave_se_data[DYN_SLAVE_SE_BUFFER_LEN];
uint8_t dyn_slave_se_num_rx_bytes;
uint8_t dyn_slave_se_num_tx_bytes;
uint8_t dyn_slave_se_packet_ready;
uint8_t dyn_slave_se_send_done;
uint8_t dyn_slave_se_error;
uint8_t dyn_slave_se_tmp_data[RAM_SIZE];
uint8_t dyn_slave_se_tmp_address;
uint8_t dyn_slave_se_tmp_length;
uint8_t dyn_slave_se_reg_pending;
uint8_t dyn_slave_se_sync_read_pending;
uint8_t dyn_slave_se_sync_read_previous;
uint8_t dyn_slave_se_bulk_read_pending;
uint8_t dyn_slave_se_bulk_read_previous;

inline void dyn_slave_se_set_rx(void)
{
  PORTD &= 0xBF;
  PORTD |= 0x80;
}

inline void dyn_slave_se_set_tx(void)
{
  PORTD &= 0x7F;
  PORTD |= 0x40;
}

/* interrupt handlers */
ISR(USART1_TX_vect)
{
  if(dyn_slave_se_num_tx_bytes==dyn_slave_se_data[3]+4)
  {
    dyn_slave_se_set_rx();
    dyn_slave_se_send_done=1;
  }
  else
  {
    UDR1=dyn_slave_se_data[dyn_slave_se_num_tx_bytes];
    dyn_slave_se_num_tx_bytes++;
  }
}

ISR(USART1_RX_vect)
{
  static uint8_t length;

  cli();// disable any other interrupt
  dyn_slave_se_data[dyn_slave_se_num_rx_bytes]=UDR1;
  switch(dyn_slave_se_num_rx_bytes)
  {
    case 0: if(dyn_slave_se_data[dyn_slave_se_num_rx_bytes]==0xFF)
              dyn_slave_se_num_rx_bytes++;
            break;
    case 1: if(dyn_slave_se_data[dyn_slave_se_num_rx_bytes]==0xFF)
              dyn_slave_se_num_rx_bytes++;
            else
              dyn_slave_se_num_rx_bytes--;
            break;
    case 2: dyn_slave_se_num_rx_bytes++;
            break;
    case 3: length=dyn_slave_se_data[dyn_slave_se_num_rx_bytes]+3;
            dyn_slave_se_num_rx_bytes++;
            break;
    default: if(dyn_slave_se_num_rx_bytes==length)
             {
               dyn_slave_se_num_rx_bytes=0;
               dyn_slave_se_packet_ready=1;
             }
             else
               dyn_slave_se_num_rx_bytes++;
             break;
  }
  sei();// enable all interrutps
}

/* private functions */
uint8_t dyn_slave_se_is_packet_ready(void)
{
  if(dyn_slave_se_packet_ready==0x01)
  {
    dyn_slave_se_packet_ready=0x00;
    return 0x01;
  }
  else
    return 0x00;
}

uint8_t dyn_slave_se_check_id(void)
{
  if(dyn_slave_se_id==dyn_slave_se_data[2] || dyn_slave_se_data[2]==0xFE)
    return 0x01;
  else
  { 
    dyn_slave_se_num_rx_bytes=0;
    return 0x00;
  }
}

uint8_t dyn_slave_se_get_id(void)
{
  return dyn_slave_se_data[2];
}

uint8_t dyn_slave_se_check_checksum(void)
{
  uint8_t i,cs=0x00;

  for(i=2;i<dyn_slave_se_data[3]+4;i++)
    cs+=dyn_slave_se_data[i];
  if(cs==0xFF)
    return 0x01;
  else 
    return 0x00;
}

inline uint8_t dyn_slave_se_get_instruction(void)
{
  return dyn_slave_se_data[4];
}

inline uint8_t dyn_slave_se_get_received_id(void)
{
  return dyn_slave_se_data[2];
}

// read operation
inline uint8_t dyn_slave_se_get_address(void)
{
  return dyn_slave_se_data[5];
}

inline uint8_t dyn_slave_se_get_read_length(void)
{
  return dyn_slave_se_data[6];
}

// write operation
inline uint8_t dyn_slave_se_get_write_length(void)
{
  return dyn_slave_se_data[3]-3;
}

inline uint8_t *dyn_slave_se_get_write_data(void)
{
  return &dyn_slave_se_data[6];
}

// sync read operation
uint8_t dyn_slave_se_sync_read_id_present(void)
{
  uint8_t i,num;

  num=dyn_slave_se_data[3]-4;
  for(i=0;i<num;i++)
  {
    if(dyn_slave_se_data[7+i]==dyn_slave_se_id)
    {
      if(i==0)
        return 0x00;
      else
        return dyn_slave_se_data[6+i];
    }
  }
  return 0xFF;
}

// sync write operation
uint8_t dyn_slave_se_sync_write_id_present(uint8_t **data)
{
  uint8_t i,num;

  num=(dyn_slave_se_data[3]-4)/(dyn_slave_se_data[6]+1);
  for(i=0;i<num;i++)
  {
    if(dyn_slave_se_data[7+i*(dyn_slave_se_data[6]+1)]==dyn_slave_se_id)
    {
      (*data)=&dyn_slave_se_data[7+i*(dyn_slave_se_data[6]+1)+1];
      return 0x01;
    }
  }
  return 0x00;
}

// bulk read operation
uint8_t dyn_slave_se_bulk_read_id_present(uint8_t *address,uint8_t *length)
{
  uint8_t i,num;

  num=(dyn_slave_se_data[3]-2)/3;
  for(i=0;i<num;i++)
  {
    if(dyn_slave_se_data[5+i*3]==dyn_slave_se_id)
    {
      (*address)=dyn_slave_se_data[6+i*3];
      (*length)=dyn_slave_se_data[7+i*3];
      if(i==0)
        return 0x00;
      else
        return dyn_slave_se_data[5+(i-1)*3];
    }
  }
  return 0xFF;
}

// bulk write operation
uint8_t dyn_slave_se_bulk_write_id_present(uint8_t *address,uint8_t *length,uint8_t **data)
{
  uint8_t i,num;

  num=dyn_slave_se_data[3]+4;
  for(i=5;i<num;)
  {
    if(dyn_slave_se_data[i]==dyn_slave_se_id)
    {
      (*address)=dyn_slave_se_data[i+1];
      (*length)=dyn_slave_se_data[i+2];
      (*data)=&dyn_slave_se_data[i+3];
      return 0x01;
    }
    else
      i+=dyn_slave_se_data[i+2]+3;
  }
  return 0x00;
}

void dyn_slave_se_send_status(uint8_t error,uint8_t length, uint8_t *data)
{
  uint8_t i,cs=0;

  if(dyn_slave_se_data[2]!=0xFE)
  {
    if(dyn_slave_se_data[4]==INST_PING || ((dyn_slave_se_data[4]==INST_READ || 
       dyn_slave_se_data[4]==INST_SYNC_READ || dyn_slave_se_data[4]==INST_BULK_READ) && 
       ram_data[Status_Return_Level]!=0x00) || ram_data[Status_Return_Level]==0x02) 
    dyn_slave_se_data[0]=0xFF;
    dyn_slave_se_data[1]=0xFF;
    dyn_slave_se_data[2]=dyn_slave_se_get_id();
    cs+=dyn_slave_se_get_id();
    dyn_slave_se_data[3]=length+2;
    cs+=length+2;
    dyn_slave_se_data[4]=error;
    cs+=error;
    for(i=0;i<length;i++)
    {
      dyn_slave_se_data[5+i]=data[i];
      cs+=data[i];
    }
    dyn_slave_se_data[5+i]=~cs;
    // set in tex mode
    dyn_slave_se_set_tx();
    // start transmission
    dyn_slave_se_num_tx_bytes=1;
    UDR1=dyn_slave_se_data[0];
  }
}

inline uint8_t dyn_slave_se_is_send_done(void)
{
  return dyn_slave_se_send_done;
}

/* public functions */
void dyn_slave_se_init(uint8_t baudrate,uint8_t id)
{
  // DDRD - Port D Data Direction Register 
  DDRD=DDRD|0xC8;// RX_en (7), TX_en (6), TX (3) are outputs 0xC2=11000010
  DDRD=DDRD&0xFB;// RX(2) is an input     0xFE=11111110

  // configure the ports to receive data
  dyn_slave_se_set_rx();

  // initialize the rs485 ports
  UCSR1A = (1<<U2X);// double USART transmission speed
  dyn_slave_se_set_baudrate(baudrate);
  UCSR1A|=0x40;// clear any pending interrupt
  UCSR1B = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE)|(1<<TXCIE);
  UCSR1C = (1<<UCSZ1)|(1<< UCSZ0); // 8 bit data, no parity, 1 stop bit

  dyn_slave_se_set_id(id);
  // initialize private variables
  dyn_slave_se_num_tx_bytes=0;
  dyn_slave_se_num_rx_bytes=0;
  dyn_slave_se_packet_ready=0;
  dyn_slave_se_send_done=0;
  dyn_slave_se_error=NO_ERROR;
  dyn_slave_se_tmp_address=0x00;
  dyn_slave_se_tmp_length=0x00;
  dyn_slave_se_reg_pending=0x00;
  dyn_slave_se_sync_read_pending=0x00;
  dyn_slave_se_bulk_read_pending=0x00;
}

void dyn_slave_se_set_baudrate(uint8_t baudrate)
{
  UBRR1H = 0;
  UBRR1L = baudrate;
  ram_data[Baud_Rate]=baudrate;
}

void dyn_slave_se_set_id(uint8_t id)
{
  dyn_slave_se_id=id;
  ram_data[ID]=id;
}

void dyn_slave_se_loop(void)
{
  static uint8_t *data,i;
  uint8_t id;

  if(dyn_slave_se_is_packet_ready())
  {
    id=dyn_slave_se_get_id();
    if(dyn_slave_se_check_id() || pwm_dyn_check_id(id))
    {
      if(dyn_slave_se_check_checksum())
      {
        switch(dyn_slave_se_get_instruction())
        {
          case INST_PING: dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,0x00,0x00000000);
                          break;
          case INST_READ: if(dyn_slave_se_check_id())
                          {
                            if(do_read(dyn_slave_se_get_address(),dyn_slave_se_get_read_length(),&data))
                              dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,dyn_slave_se_get_read_length(),data);
                            else
                              dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000);
                          }
                          if(pwm_dyn_check_id(id))
                          {
                            if(pwm_dyn_do_read(id,dyn_slave_se_get_address(),dyn_slave_se_get_read_length(),&data))
                              dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,dyn_slave_se_get_read_length(),data);
                            else
                              dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000);
                          }
                          break;
          case INST_WRITE: if(dyn_slave_se_check_id())
                           {
                             if(do_write(dyn_slave_se_get_address(),dyn_slave_se_get_write_length(),dyn_slave_se_get_write_data()))
                               dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,0x00,0x00000000);
                             else 
                               dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000);
                           }
                           if(pwm_dyn_check_id(id))
                           {
                             if(pwm_dyn_do_write(id,dyn_slave_se_get_address(),dyn_slave_se_get_write_length(),dyn_slave_se_get_write_data()))
                               dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,0x00,0x00000000);
                             else 
                               dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000);
                           }
                           break;
          case INST_REG_WRITE: if(dyn_slave_se_check_id())
                               {
                                 dyn_slave_se_tmp_address=dyn_slave_se_get_address();
                                 dyn_slave_se_tmp_length=dyn_slave_se_get_write_length();
                                 data=dyn_slave_se_get_write_data();
                                 for(i=0;i<dyn_slave_se_tmp_length;i++)
                                   dyn_slave_se_tmp_data[i]=data[i];
                                 dyn_slave_se_reg_pending=0x01;
                               }
                               if(pwm_dyn_check_id(id))
                                 pwm_dyn_do_reg_write(id,dyn_slave_se_get_address(),dyn_slave_se_get_write_length(),dyn_slave_se_get_write_data());
                               dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,0x00,0x00000000);
                               break;
          case INST_ACTION: if(dyn_slave_se_reg_pending)
                            {
                              if(dyn_slave_se_check_id())
                                do_write(dyn_slave_se_tmp_address,dyn_slave_se_tmp_length,dyn_slave_se_tmp_data);
                              if(pwm_dyn_check_id(id))
                                pwm_dyn_do_action(id);
                            }
                            else
                              dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000);
                            break;
          case INST_RESET:
                           break;
          case INST_SYNC_READ: dyn_slave_se_sync_read_previous=dyn_slave_se_sync_read_id_present();
                               if(dyn_slave_se_sync_read_previous!=0xFF)
                               {
                                 if(dyn_slave_se_sync_read_previous==0x00)// first device on the list
                                 {
                                   if(do_read(dyn_slave_se_get_address(),dyn_slave_se_get_read_length(),&data))
                                     dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,dyn_slave_se_get_read_length(),data);
                                   else
                                     dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000);
                                 }
                                 else// sync read pending
                                 {
                                   dyn_slave_se_tmp_address=dyn_slave_se_get_address();
                                   dyn_slave_se_tmp_length=dyn_slave_se_get_read_length();
                                   dyn_slave_se_sync_read_pending=0x01;
                                 }
                               }
                               break;
          case INST_SYNC_WRITE: if(dyn_slave_se_sync_write_id_present(&data))
                                  do_write(dyn_slave_se_get_address(),dyn_slave_se_get_read_length(),data);
                                pwm_dyn_do_sync_write(dyn_slave_se_data);
                                break;
          case INST_BULK_READ: dyn_slave_se_bulk_read_previous=dyn_slave_se_bulk_read_id_present(&dyn_slave_se_tmp_address,&dyn_slave_se_tmp_length);
                               if(dyn_slave_se_bulk_read_previous!=0xFF)
                               {
                                 if(dyn_slave_se_bulk_read_previous==0x00)// first device on the list
                                 {
                                   if(do_read(dyn_slave_se_tmp_address,dyn_slave_se_tmp_length,&data))
                                     dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,dyn_slave_se_tmp_length,data);
                                   else
                                     dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000);
                                 }
                                 else// sync read pending
                                   dyn_slave_se_bulk_read_pending=0x01;
                               }
                               break;
          case INST_BULK_WRITE: if(dyn_slave_se_bulk_write_id_present(&dyn_slave_se_tmp_address,&dyn_slave_se_tmp_length,&data))
                                  do_write(dyn_slave_se_tmp_address,dyn_slave_se_tmp_length,data);
                                break;
          default: dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000);
                   break;
        }
      }
      else
        dyn_slave_se_send_status(dyn_slave_se_error|CHECKSUM_ERROR,0x00,0x00000000);
    }
    else
    {
      if(dyn_slave_se_sync_read_pending)
      {
        if(dyn_slave_se_get_received_id()==dyn_slave_se_sync_read_previous)
        {
          dyn_slave_se_sync_read_pending=0x00;
          if(do_read(dyn_slave_se_tmp_address,dyn_slave_se_tmp_length,&data))
            dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,dyn_slave_se_tmp_length,data);
          else
            dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000); 
        }
      }
      if(dyn_slave_se_bulk_read_pending)
      {
        if(dyn_slave_se_get_received_id()==dyn_slave_se_bulk_read_previous)
        {
          dyn_slave_se_bulk_read_pending=0x00;
          if(do_read(dyn_slave_se_tmp_address,dyn_slave_se_tmp_length,&data))
            dyn_slave_se_send_status(dyn_slave_se_error|NO_ERROR,dyn_slave_se_tmp_length,data);
          else
            dyn_slave_se_send_status(dyn_slave_se_error|INSTRUCTION_ERROR,0x00,0x00000000); 
        }
      }
    }
  }
}

void dyn_slave_se_set_error(uint8_t error)
{
  dyn_slave_se_error|=error;
}

void dyn_slave_se_clear_error(void)
{
  dyn_slave_se_error=NO_ERROR;
}
