#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "dyn_slave_se.h"
#include "dyn_slave_diff.h"
#include "mem.h"
#include "ports.h"
#include "adc.h"
#ifdef _PWM
#include "pwm.h"
#endif
#ifdef _DAC
#include "dac.h"
#endif
#ifdef _BNO055
#include "bno055.h"
#else
#include "compass.h"
#endif
#ifdef _UART_TTL
#include "uart_ttl.h"
#endif
#ifdef _UART_USB
#include "uart_usb.h"
#endif
#include "pwm_dyn_emm.h"

int16_t main(void) 
{
  ram_init();
  dyn_slave_se_init(ram_data[Baud_Rate],ram_data[ID]);
  dyn_slave_diff_init(ram_data[Baud_Rate],ram_data[ID]);
  init_ports();
  adc_init();
  #ifdef _PWM
  pwm_init();
  #endif
  #ifdef _DAC
  dac_init();
  #endif
  #ifdef _UART_TTL
  uart_ttl_init();
  #endif
  #ifdef _UART_USB
  uart_usb_init();
  #endif
  pwm_dyn_init();
  sei();
  #ifdef _BNO055
  bno055_init();
  #else
  compass_init();
  #endif

  while (1) 
  {
    ports_loop();
    adc_loop();
    #ifdef _BNO055
    bno055_loop();
    #else
    compass_loop();
    #endif
    dyn_slave_se_loop();
    dyn_slave_diff_loop();
    pwm_dyn_loop();
  }
}
