#include "mem.h"
#include <avr/eeprom.h>

// dynamixel RAM variables
uint8_t ram_data[RAM_SIZE];

#ifdef _BNO055
  #ifdef _DAC
    #ifdef _PWM
      #define PERIPHERALS      0x13
    #endif
    #ifdef _UART_TTL
      #define PERIPHERALS      0x16
    #endif
  #endif
  #ifdef _UART_USB
    #ifdef _PWM
      #define PERIPHERALS      0x19
    #endif
    #ifdef _UART_TTL
      #define PERIPHERALS      0x1C
    #endif
  #endif 
#else
  #ifdef _DAC
    #ifdef _PWM
      #define PERIPHERALS      0x03
    #endif
    #ifdef _UART_TTL
      #define PERIPHERALS      0x06
    #endif
  #endif
  #ifdef _UART_USB
    #ifdef _PWM
      #define PERIPHERALS      0x09
    #endif
    #ifdef _UART_TTL
      #define PERIPHERALS      0x0C
    #endif
  #endif 
#endif

// Dynamixel EEPROM variables
uint8_t EEMEM eeprom_data[17]={0x1C,0x00,0x00,0x1E,0x01,0xFA,PERIPHERALS,0x13,0x14,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x02};

void ram_init(void)
{
  uint8_t i;

  for(i=0;i<=17;i++)
    ram_data[i]=eeprom_read_byte(&eeprom_data[i]);
  for(;i<RAM_SIZE;i++)
    ram_data[i]=0x00;
}

uint8_t ram_read(uint8_t address, uint8_t length, uint8_t **data)
{
  if((address+length)>RAM_SIZE)
    return 0x00;
  else
  {
    (*data)=&ram_data[address];
    return 0x01;
  }
}

uint8_t ram_write(uint8_t address, uint8_t length, uint8_t *data)
{
  uint8_t i,num=0;

  if((address+length)>RAM_SIZE)
    return 0x00;
  else
  {
    for(i=address;i<address+length;i++)
    {
      ram_data[i]=data[num];
      num++;
    }
    return 0x01;
  }
}

uint8_t eeprom_write(uint8_t address, uint8_t length, uint8_t *data)
{
  uint8_t i,num=0;

  if((address+length)>RAM_SIZE)
    return 0x00;
  else
  {
    for(i=address;i<address+length;i++)
    {
      if(i<=17)
        eeprom_write_byte(&eeprom_data[i],data[num]);
      num++;
    }
    return 0x01;
  }
}
