#include <avr/io.h>
#include "ports.h"
#include "mem.h"

TGPIO gpio_bits[NUM_GPIO]={{&DDRA,&PORTA,&PINA,0},
                           {&DDRA,&PORTA,&PINA,1},
                           {&DDRA,&PORTA,&PINA,2},
                           {&DDRA,&PORTA,&PINA,3},
                           {&DDRA,&PORTA,&PINA,4},
                           {&DDRA,&PORTA,&PINA,5},
                           {&DDRA,&PORTA,&PINA,6}, 
                           {&DDRA,&PORTA,&PINA,7},
                           {&DDRG,&PORTG,&PING,2},
                           {&DDRC,&PORTC,&PINC,7},
                           {&DDRC,&PORTC,&PINC,6},
                           {&DDRC,&PORTC,&PINC,5},
                           {&DDRC,&PORTC,&PINC,4},
                           {&DDRC,&PORTC,&PINC,3},
                           {&DDRC,&PORTC,&PINC,2},
                           {&DDRC,&PORTC,&PINC,1},
                           {&DDRC,&PORTC,&PINC,0},
                           {&DDRG,&PORTG,&PING,1},
                           {&DDRG,&PORTG,&PING,0}};

void init_ports(void)
{
  //auxiliar LED
  DDRE|=0x04;
  clear_led();

  // general purpose input/output
  // all inputs by default
  DDRA=0x00;
  DDRC=0x00;
  DDRG&=0xF8;

  // input selector
  DDRG&=0xE7;
}

void config_port(uint8_t port_id,uint8_t dir)
{
  if(port_id>=0 && port_id<NUM_GPIO)
  {
    if(dir==OUTPUT)// configure as an output
    {
      *(gpio_bits[port_id].dir_reg)|=0x01<<gpio_bits[port_id].bit;
      ram_data[GPIO0_config+port_id*2]=0x01;
    }
    else// configure as an input
    {
      *(gpio_bits[port_id].dir_reg)&=~(0x01<<gpio_bits[port_id].bit);
      ram_data[GPIO0_config+port_id*2]=0x00;
    }
  }
}

uint8_t get_port_config(uint8_t port_id)
{
  uint8_t dir;

  if(port_id>=0 && port_id<NUM_GPIO)
  {
    dir=(*(gpio_bits[port_id].dir_reg)&(0x01<<gpio_bits[port_id].bit));
    if(dir==0x00)
      return INPUT;
    else
      return OUTPUT;
  }
  else
    return INPUT;
}

void set_port(uint8_t port_id)
{
  if(port_id>=0 && port_id<NUM_GPIO)
  {
    *(gpio_bits[port_id].write_reg)|=0x01<<gpio_bits[port_id].bit;
    ram_data[GPIO0_data+port_id*2]=0x01;
    ram_data[GPIO0_data_block+port_id]=0x01;
  }
}

void clear_port(uint8_t port_id)
{
  if(port_id>=0 && port_id<NUM_GPIO)
  {
    *(gpio_bits[port_id].write_reg)&=~(0x01<<gpio_bits[port_id].bit);
    ram_data[GPIO0_data+port_id*2]=0x00;
    ram_data[GPIO0_data_block+port_id]=0x00;
  }
}

uint8_t read_port(uint8_t port_id)
{
  uint8_t value;
  if(port_id>=0 && port_id<NUM_GPIO)
  {
    value=((*(gpio_bits[port_id].read_reg))&(0x01<<gpio_bits[port_id].bit));
    if(value==0x00)
    {
      ram_data[GPIO0_data+port_id*2]=0x00;
      return 0x00;
    }
    else
    {
      ram_data[GPIO0_data+port_id*2]=0x01;
      return 0x01;
    }
  }
  else
    return 0x00;
}

void set_led(void)
{
  PORTE&=0xFB;
  ram_data[LED]=0x01;
}

void clear_led(void)
{
  PORTE|=0x04;
  ram_data[LED]=0x00;
}

void toggle_led(void)
{
  PORTE^=0x04;
  ram_data[LED]^=0x01;
}

uint8_t get_selector_value(void)
{
  ram_data[SWITCHES]=(PING&0x18)>>3;
  return ((PING&0x18)>>3);
}

void ports_loop(void)
{
  uint8_t i,value;

  for(i=0;i<NUM_GPIO;i++)
  {
    value=((*(gpio_bits[i].read_reg))&(0x01<<gpio_bits[i].bit));
    if(value==0x00)
      ram_data[GPIO0_data_block+i]=0x00;
    else
      ram_data[GPIO0_data_block+i]=0x01;
  }
}
