#include "pwm_dyn_emm.h"
#include "dyn_servos_reg.h"
#include "mem.h"
#include "pwm.h"

// private variables
uint8_t pwm_dyn_data[4][50];
// variables for the registered write
uint8_t pwm_dyn_data_tmp[4][50];
uint8_t pwm_dyn_reg_write_addr[4];
uint8_t pwm_dyn_reg_write_len[4];
uint8_t pwm_dyn_reg_write_pending[4];

uint16_t pwm_dyn_current_duty[4];
uint16_t pwm_dyn_target_duty[4];
uint16_t pwm_dyn_target_speed[4];

//private functions
uint8_t pwm_dyn_sync_write_id_present(uint8_t id, uint8_t *buffer,uint8_t **data)
{
  uint8_t i,num;

  num=(buffer[3]-4)/(buffer[6]+1);
  for(i=0;i<num;i++)
  {
    if(buffer[7+i*(buffer[6]+1)]==id)
    {
      (*data)=&buffer[7+i*(buffer[6]+1)+1];
      return 0x01;
    }
  }
  return 0x00;
}

// public functions
void pwm_dyn_init(void)
{
  uint8_t i,j,num=0;

  //start the PWM module
  pwm_start();
  pwm_set_frequency(50);//50 Hz

  for(i=0;i<4;i++)
  {
    for(j=0;j<50;j++)
    {
      pwm_dyn_data[i][j]=0x00;
      pwm_dyn_data_tmp[i][j]=0x00;
    }
    // initialize emmulated servo data
    pwm_dyn_data[i][SERVO_ID]=ram_data[PWM0_DYN_ID+i];
    pwm_dyn_data[i][MODEL_NUM]=0x00;
    pwm_dyn_data[i][MODEL_NUM+1]=0x0C;
    pwm_dyn_data[i][BAUD_RATE]=0x01;
    pwm_dyn_data[i][RET_DEL_TIME]=0xFA;
    pwm_dyn_data[i][CW_ANGLE_LIM]=0x00;// limit to 60 degrees
    pwm_dyn_data[i][CW_ANGLE_LIM+1]=0x00;
    pwm_dyn_data[i][CCW_ANGLE_LIM]=0xFF;// limit to 240 degrees
    pwm_dyn_data[i][CCW_ANGLE_LIM+1]=0x03;
    pwm_dyn_data[i][RET_DEL_TIME]=0x02;
    pwm_dyn_data[i][TEMP_LIM]=0x46;
    pwm_dyn_data[i][LOW_VOLTAGE_LIM]=0x3C;
    pwm_dyn_data[i][HIGH_VOLTAGE_LIM]=0xBE;
    pwm_dyn_data[i][MAX_TORQUE]=0xFF;
    pwm_dyn_data[i][MAX_TORQUE+1]=0x03;
    pwm_dyn_data[i][CW_COMP_MARGIN]=0x01;
    pwm_dyn_data[i][CCW_COMP_MARGIN]=0x01;
    pwm_dyn_data[i][CW_COMP_SLOPE]=0x20;
    pwm_dyn_data[i][CCW_COMP_SLOPE]=0x20;
    pwm_dyn_data[i][PUNCH]=0x20;

    pwm_dyn_data[i][CURRENT_POS]=0xFF;
    pwm_dyn_data[i][CURRENT_POS+1]=0x01;
    pwm_dyn_data[i][CURRENT_SPEED]=0x00;
    pwm_dyn_data[i][CURRENT_SPEED+1]=0x00;

    pwm_dyn_reg_write_addr[i]=0x00;
    pwm_dyn_reg_write_len[i]=0x00;
    pwm_dyn_reg_write_pending[i]=0x00;

    // enable the PWM channel if ID!=0xFF
    if(ram_data[PWM0_DYN_ID+i]!=0xFF)
    {
      pwm_enable_ch((pwm_id_t)i);
      pwm_set_duty_cycle((pwm_id_t)i,3840);
      num++;
    }
    pwm_dyn_current_duty[i]=3840;
    pwm_dyn_target_duty[i]=3840;
    pwm_dyn_target_speed[i]=5000;
  }
  ram_data[NUM_PWM_SERVOS]=num;
}

void pwm_dyn_set_dyn_id(uint8_t index,uint8_t new_id)
{
  if(pwm_dyn_data[index][ID]==0xFF && new_id!=0xFF)
    ram_data[NUM_PWM_SERVOS]++;
  else if(pwm_dyn_data[index][ID]!=0xFF && new_id==0xFF)
    ram_data[NUM_PWM_SERVOS]--;
  pwm_dyn_data[index][ID]=new_id;
  if(new_id==0xFF)
    pwm_disable_ch((pwm_id_t)index);
  else
    pwm_enable_ch((pwm_id_t)index);
}

uint8_t pwm_dyn_check_id(uint8_t id)
{
  uint8_t i;

  if(id==0xFE) return 0x01;// broadcast
  for(i=0;i<4;i++)
    if(pwm_dyn_data[i][SERVO_ID]==id)
      return 0x01;

  return 0x00;
}

uint8_t pwm_dyn_do_write(uint8_t id,uint8_t address,uint8_t length,uint8_t *data)
{
  uint8_t i,j;
  uint16_t duty,speed;

  for(i=0;i<4;i++)
  {
    if(pwm_dyn_data[i][SERVO_ID]==id || id==0xFE)
    {
      for(j=0;j<length;j++)
        pwm_dyn_data[i][address+j]=data[j];
      // do any required action
      if(address<=GOAL_POS && address+length>=GOAL_POS+1)// target position
      {
        duty=((uint16_t)pwm_dyn_data[i][GOAL_POS]+(uint16_t)(pwm_dyn_data[i][GOAL_POS+1]<<8));
        duty=(((uint32_t)duty*1800)>>10)+600;
        duty=((uint32_t)duty*100*512)/20000;
        pwm_dyn_target_duty[i]=duty;
      }
      if(address<=GOAL_SPEED && address+length>=GOAL_SPEED+1)
      {
        speed=((uint16_t)pwm_dyn_data[i][GOAL_SPEED]+(uint16_t)(pwm_dyn_data[i][GOAL_SPEED+1]<<8));
        if(speed==0)
          pwm_dyn_target_speed[i]=5000;
        else
        {
          speed=((uint32_t)speed*6000)/1024;//dynamixel digital to degrees/s
          speed=((uint32_t)speed*100*512)/((uint32_t)20000*(uint32_t)50);
          pwm_dyn_target_speed[i]=speed;
        }
      }
    }
  }  

  return 0x01;
}

uint8_t pwm_dyn_do_read(uint8_t id,uint8_t address,uint8_t length,uint8_t **data)
{
  uint8_t i;

  for(i=0;i<4;i++)
    if(pwm_dyn_data[i][SERVO_ID]==id)
      (*data)=&pwm_dyn_data[i][address];

  return 0x01;
}

uint8_t pwm_dyn_do_reg_write(uint8_t id,uint8_t address, uint8_t length,uint8_t *data)
{
  uint8_t i,j;

  for(i=0;i<4;i++)
  {
    if(pwm_dyn_data[i][SERVO_ID]==id || id==0xFE)
    {
      for(j=0;j<length;j++)
        pwm_dyn_data_tmp[i][address+j]=data[j];
      pwm_dyn_reg_write_addr[i]=address;
      pwm_dyn_reg_write_len[i]=length;
      pwm_dyn_reg_write_pending[i]=0x01;
    }
  }  

  return 0x01;
}

uint8_t pwm_dyn_do_action(uint8_t id)
{
  uint8_t i;

  for(i=0;i<4;i++)
  {
    if(pwm_dyn_data[i][SERVO_ID]==id || id==0xFE) 
    {
      if(pwm_dyn_reg_write_pending[i]==0x01)
      {
        pwm_dyn_do_write(id,pwm_dyn_reg_write_addr[i],pwm_dyn_reg_write_len[i],&pwm_dyn_data_tmp[i][pwm_dyn_reg_write_addr[i]]);
        pwm_dyn_reg_write_addr[i]=0x00;
        pwm_dyn_reg_write_len[i]=0x00;
        pwm_dyn_reg_write_pending[i]=0x00;
      }
    }
  }

  return 0x01;
}

uint8_t pwm_dyn_do_sync_write(uint8_t *buffer)
{
  uint8_t i,*data;

  for(i=0;i<4;i++)
  {
    if(pwm_dyn_sync_write_id_present(pwm_dyn_data[i][SERVO_ID],buffer,&data))
      pwm_dyn_do_write(pwm_dyn_data[i][SERVO_ID],buffer[5],buffer[6],data);
  }

  return 0x01;
}

void pwm_dyn_loop(void)
{
  uint8_t i;

  // wait for the TIM3 interrupt (PWM timer) 
  if(ETIFR&TOV3)
  {
    ETIFR&=TOV3;// clear interrupt flag
    for(i=0;i<4;i++)
    {
      if(pwm_dyn_current_duty[i]<pwm_dyn_target_duty[i])
      {
        pwm_dyn_current_duty[i]+=pwm_dyn_target_speed[i];
        if(pwm_dyn_current_duty[i]>pwm_dyn_target_duty[i])
          pwm_dyn_current_duty[i]=pwm_dyn_target_duty[i];
      }
      else if(pwm_dyn_current_duty[i]>pwm_dyn_target_duty[i])
      {
        if(pwm_dyn_current_duty[i]<pwm_dyn_target_speed[i])
          pwm_dyn_current_duty[i]=pwm_dyn_target_duty[i];
        else
        {
          pwm_dyn_current_duty[i]-=pwm_dyn_target_speed[i];
          if(pwm_dyn_current_duty[i]<pwm_dyn_target_duty[i])
            pwm_dyn_current_duty[i]=pwm_dyn_target_duty[i];
        }
      }
      // else, do nothing
      pwm_set_duty_cycle((pwm_id_t)i,pwm_dyn_current_duty[i]);
    }
  }
}
